pub mod pk {

    use std::collections::HashMap;

    use reqwest::header::{USER_AGENT, AUTHORIZATION};

    use serde_json::Value;

    use crate::api::request::*;
    use crate::systemdata::*;

    use color_eyre::eyre::{eyre, Result};

    pub const PK_URL: &str = "https://api.pluralkit.me/v2";

    pub fn get_system(key: &str) -> Result<Value> {
        let url = format!("{}/systems/@me", PK_URL);

        let response = http_get(url,key)?;

        Ok(serde_json::from_str(&response)?)
    }

    pub fn get_members(key: &str, sysid: &str) -> Result<Vec<Value>> {
        let url = format!("{}/systems/{}/members", PK_URL, sysid);

        let response = http_get(url,key)?;

        let values: Vec<Value> = serde_json::from_str(&response)?;

        Ok(values)
    }

    pub fn get_fronters(key: &str, sys: &System) -> Result<Vec<Member>> {
        let url = format!("{}/systems/{}/fronters", PK_URL, sys.pk_userid);

        let result = http_get(url,key)?;
        let json: Value = serde_json::from_str(&result).unwrap();
        let json_members = &json["members"].as_array();

        let mut members: Vec<Member> = Vec::new();
        for member in json_members {
            for m in member.into_iter() {
                for dbmem in &sys.members {
                    if m["name"].as_str().unwrap().to_string() == dbmem.name {
                        members.push(dbmem.clone());
                    }
                }
            }
        }

        Ok(members)
    }


    pub fn set_fronters(key: &str, sys: &System, to_front: &Vec<Member>, fronters: &Fronters) -> Result<()> {
        let url = format!("{}/systems/{}/switches", PK_URL, sys.pk_userid);

        if to_front != &fronters.pk {
            let mut frontcodes = Vec::new();
            for tf in to_front {
                frontcodes.push(String::from(&tf.pk_id));
            }

            let mut body: HashMap<&str, Vec<String>> = HashMap::new();
            body.insert("members", frontcodes);

            let client = reqwest::Client::new();
            let rb = client
                .post(url)
                .json(&body)
                .header("content-type", "application/json; charset=utf-8")
                .header(USER_AGENT, "Pluralsync")
                .header(AUTHORIZATION, key);

            match http_request(rb) {
                Ok(_) => Ok(()),
                Err(e) => Err(eyre!("{}", e.to_string())),
            }

        } else {
            println!("Members already fonting");
            Ok(())
        }
    }
}

pub mod sp {

    use std::collections::HashMap;

    use reqwest::header::{USER_AGENT, AUTHORIZATION};

    use serde::Serialize;
    use serde_json::Value;
    use crate::api::request::*;
    use crate::systemdata::*;

    use color_eyre::eyre::{eyre, Result};

    pub const SP_URL: &str = "https://api.apparyllis.com/v1";

    pub fn get_user_id(key: &str) -> Result<String> {
        let url = format!("{}/me", SP_URL);

        let response = http_get(url, key)?;

        let json: Value = serde_json::from_str(&response)?;

        Ok(json["id"].as_str().unwrap().to_string())
    }

    pub fn get_member_ids(key: &str, system_id: &str) -> Result<HashMap<String, String>> {
        let url = format!("{}/members/{}", SP_URL, system_id);

        let response = http_get(url,key)?;
        let json: Vec<Value> = serde_json::from_str(&response)?;

        let mut sp_member_data: HashMap<String, String> = HashMap::new();
        for member in json {
            sp_member_data.insert(
                member["content"]["name"].as_str().unwrap().to_string(),
                member["id"].as_str().unwrap().to_string());
        }

        Ok(sp_member_data)
    }

    pub fn get_member_id(m: &Member, ids: &HashMap<String, String>) -> Result<String> {

        for (sp_name, sp_id) in ids {
            if &m.name == sp_name || m.aliases.iter().any(|e| sp_name.contains(e)) {
                return Ok(sp_id.to_string());
            }
        }

        Err(eyre!(format!("SP ID not found for {}", &m.name)))
    }

    pub fn get_fronters(key: &str, sys: &System) -> Result<Vec<Member>> {
        let url = format!("{}/fronters", SP_URL);

        let result = http_get(url,key)?;
        let json: Vec<Value> = serde_json::from_str(&result)?;

        let mut members = Vec::new();
        for data in json {
            let sp_id = &data["content"]["member"].as_str().unwrap().to_string();
            for member in &sys.members {
                if &member.sp_id == sp_id {
                    members.push(member.clone());
                }
            }

        }
        Ok(members)
    }

    pub fn get_front_id(key: &str, to_id: &Member) -> Result<String> {
        let url = format!("{}/fronters", SP_URL);

        let result = http_get(url,key)?;
        let datas: Vec<Value> = serde_json::from_str(&result)?;

        for data in datas {
            let sp_f_id = &data["id"].to_string();
            let sp_id = &data["content"]["member"].to_string();
            if sp_id.to_string() == to_id.sp_id {
                return Ok(sp_f_id.to_string());
            }
        }
        Err(eyre!("SP ID not found"))
    }

    pub fn set_fronters(key: &str, to_front: &Vec<Member>, fronters: &Fronters) -> Result<()> {
        if to_front == &fronters.sp {
            println!("Members already fonting");
            return Ok(());
        }

        for fronting_member in &fronters.sp {
            if !to_front.contains(&fronting_member) {
                let f_id = get_front_id(&key, &fronting_member)?;

                let url = format!("{}/frontHistory/{}", SP_URL, f_id);

                #[derive (Serialize)]
                #[allow (non_snake_case)]
                struct SpRem {
                    live: bool,
                    endTime: u64
                }

                let end_time = std::time::SystemTime::now().duration_since(std::time::SystemTime::UNIX_EPOCH)?.as_secs();
                let rem = SpRem {
                    live: false,
                    endTime: end_time * 1000
                };

                let body = serde_json::to_string(&rem)?;


                let client = reqwest::Client::new();
                let rb = client
                    .patch(url)
                    .body(body)
                    .header("content-type", "application/json; charset=utf-8")
                    .header(USER_AGENT, "Pluralsync")
                    .header(AUTHORIZATION, key);

                match http_request(rb) {
                    Ok(_) => (),
                    Err(e) => println!("{}", e.to_string()),
                }
            }
        }

        for tf_member in to_front {
            if !fronters.sp.contains(&tf_member) {
                let url = format!("{}/frontHistory", SP_URL);

                #[derive (Serialize)]
                #[allow (non_snake_case)]
                struct SpAdd {
                    member: String,
                    custom: bool,
                    live: bool,
                    startTime: u64
                }

                let start_time = std::time::SystemTime::now().duration_since(std::time::SystemTime::UNIX_EPOCH)?.as_secs();
                let rem = SpAdd {
                    member: String::from(&tf_member.sp_id),
                    custom: false,
                    live: true,
                    startTime: start_time * 1000
                };

                let body = serde_json::to_string(&rem)?;

                let client = reqwest::Client::new();
                let rb = client
                    .post(url)
                    .body(body)
                    .header("content-type", "application/json; charset=utf-8")
                    .header(USER_AGENT, "Pluralsync")
                    .header(AUTHORIZATION, key);

                match http_request(rb) {
                    Ok(_) => (),
                    Err(e) => println!("{}", e.to_string()),
                }
            }
        }
        Ok(())
    }
}

pub mod request {

    use reqwest::RequestBuilder;
    use reqwest::header::{USER_AGENT, AUTHORIZATION};
    #[cfg(feature = "fedi")]
    use reqwest::multipart::{Part, Form};

    use color_eyre::eyre::Result;


    #[tokio::main]
    pub async fn http_get(url: String, key: &str) -> Result<String> {
        let client = reqwest::Client::new();
        let rb = client
            .get(url)
            .header(USER_AGENT, "Pluralsync")
            .header(AUTHORIZATION, key);
        let res = rb.send()
            .await?
            .text()
            .await?;

        Ok(res)
    }

    #[tokio::main]
    pub async fn http_request(rb: RequestBuilder) -> Result<String> {
        let res = rb.send()
            .await?
            .text()
            .await?;
        Ok(res)
    }
}
