use crate::configdata::*;
use crate::systemdata::*;
use crate::clap_ps::ForceFrom;

use color_eyre::eyre::{eyre, Result};

pub enum CurrentScreen {
    Dash,
    Configuration,
    Exit,
}

pub enum ConfigEdit {
    PkKey,
    SpKey,
    AvatarEnabled,
    AvatarFolder,
    AvatarOutput,
    AvatarBlacklist,
    DiscEnabled,
    DiscToken,
    FediEnabled,
    FediInstance,
    FediToken,
}

pub struct App {
    pub input: String,
    pub cfg_dir: String,
    pub cfg: Option<Config>,
    pub sys: Option<System>,
    pub fronters: Option<Fronters>,
    pub current_screen: Option<CurrentScreen>,
    pub editing: Option<ConfigEdit>,
}

impl App {
    pub fn new(confpath: String) -> App {
        App {
            input: String::new(),
            cfg_dir: confpath,
            cfg: None,
            sys: None,
            fronters: None,
            current_screen: None,
            editing: None,
        }
    }

    pub fn load_system(&mut self, update: bool) -> Result<()> {

        match System::get_system(&self, update) {
            Ok(s) => {
                self.sys = Some(s);
                Ok(())
            },
            Err(e) => {
                self.sys = None;
                Err(e)
            }
        }
    }

    pub fn load_config(&mut self) -> Result<()> {

        match Config::get_config(&self.cfg_dir) {
            Ok(c) => {
                self.cfg = Some(c);
                Ok(())
            },
            Err(e) => {
                self.cfg = None;
                Err(e)
            },
        }

    }

    fn get(&mut self, ff: ForceFrom) -> Result<()> {

        self.fronters = Some(Fronters::get(self, ff)?);

        Ok(())
    }

    fn set(&mut self, to_front: Vec<Member>) -> Result<()> {

        self.get(ForceFrom::None)?;

        self.fronters = Some(Fronters::set(self, to_front)?);

        Ok(())
    }

    fn add(&mut self, to_front: Vec<Member>) -> Result<()> {

        let aux_fronters: Fronters = Fronters::get(self, ForceFrom::None)?;

        let mut addfronters: Vec<Member> = Vec::new();
        addfronters.append(&mut aux_fronters.pk.clone());
        addfronters.append(&mut to_front.clone());

        self.fronters = Some(Fronters::set(self, addfronters)?);

        Ok(())
    }

    pub fn memberlist(&self) -> Result<()> {

        if let Some(sys) = &self.sys {
            for member in &sys.members {
                println!("{} / {}", member.name, member.aliases.join(" | "));
            }
        }

        Ok(())
    }

    pub fn set_member(&mut self, tf_members: Vec<String>, append: bool) -> Result<()> {

        if let Some(sys) = &self.sys {
            let mut to_front: Vec<Member> = Vec::new();
            for member in &tf_members {
                for mem in &sys.members {
                    let lowcase_aliases: Vec<String> = mem.aliases.iter().map(|x| x.to_lowercase()).collect();
                    if mem.name.to_lowercase() == member.to_lowercase() || lowcase_aliases.iter().any(|e| member.contains(e)) {
                        println!("Member {member} found.");

                        to_front.push(mem.clone());

                        break;
                    }
                }
            }

            if to_front.len() == tf_members.len() {
                if !append {
                    self.set(to_front)?;
                } else {
                    self.add(to_front)?;
                }
            } else {
                println!("One or more members were not found. Known members:\n--------------------------");
                self.memberlist()?;
                println!("--------------------------\nIf a member is missing from the system try running \"pluralsync update\" to refresh the local database");
                return Err(eyre!("Missing member"));
            }

            #[cfg(feature = "jlog")] {
                let mut names = Vec::new();
                for m in &to_front {
                    names.push(String::from(&m.name));
                }
                let log_fronters = names.join(" || ");

                #[cfg(target_os = "windows")]
                std::process::Command::new("jlog").args(["info", format!("Switch registered: {}", log_fronters)]).output().expect("Logging error");

                #[cfg(not(target_os = "windows"))]
                std::process::Command::new("jlog").arg("info").arg(format!("Switch registered: {}", log_fronters)).output().expect("Logging error");
            }

            return Ok(());
        }
        Err(eyre!("No system found"))
    }

    pub fn get_front(&mut self, ff: ForceFrom) -> Result<Vec<String>> {

        self.get(ff)?;

        if let Some(fronters) = &self.fronters {
            let mut names: Vec<String> = Vec::new();
            for member in &fronters.pk {
                names.push(String::from(&member.name));
            }

            let fronters = names.join(" || ");
            println!("Currently fronting: {}", fronters);
            std::fs::write(format!("{}/.front", &self.cfg_dir), &fronters)?;

            return Ok(names);
        }
        Err(eyre!("No fronters found"))
    }
}
